<?php

namespace Drupal\make\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\make\Service\MakeAsApiService;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Messenger\MessengerInterface;

class MakeAsSubscribeConfigForm extends ConfigFormBase {

  /** @var \Drupal\make\Service\MakeAsApiService */
  protected $makeAsApiService;

  /** @var \Drupal\Core\Messenger\MessengerInterface */
  protected $messenger;

  /**
   * Constructor for MakeAsSubscribeConfigForm.
   *
   * @param \Drupal\make\Service\MakeAsApiService $make_as_api_service
   *   The Make API service.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   */
  public function __construct(MakeAsApiService $make_as_api_service, MessengerInterface $messenger) {
    $this->makeAsApiService = $make_as_api_service;
    $this->messenger = $messenger;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('make.api_service'),
      $container->get('messenger')
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return ['make.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'make_config_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = [
      '#type' => 'vertical_tabs',
      '#default_tab' => 'edit-authentication',
    ];

    // Authentication Section
    $form['authentication'] = [
      '#type' => 'details',
      '#title' => $this->t('Authentication'),
      '#group' => 'tabs',
      '#description' => $this->t('Enter your Make <strong>User ID</strong> and <strong>API Key</strong> to authenticate with the Make API. Get your User ID and API key from <a href="https://newsletter.make.as/#/settings/general/integrations" target="_blank">Integrations & API</a> section in Make.'),
      '#open' => TRUE,
    ];

    $form['authentication']['user_id'] = [
      '#type' => 'textfield',
      '#title' => $this->t('User ID'),
      '#default_value' => $this->config('make.settings')->get('user_id'),
      '#required' => TRUE,
    ];

    $form['authentication']['api_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('API Key'),
      '#default_value' => $this->config('make.settings')->get('api_key'),
      '#required' => TRUE,
    ];

    // Subscribers List Section
    $form['subscribers_list'] = [
      '#type' => 'details',
      '#title' => $this->t('Subscribers List'),
      '#group' => 'tabs',
      '#description' => $this->t('Once authenticated, you can get a list of your subscriber lists here and select which one you\'d like to use.'),
      '#open' => TRUE,
    ];

    try {
      $lists = $this->makeAsApiService->getSubscriberLists();

      if (!empty($lists)) {
        $list_options = [];
        foreach ($lists as $list) {
          $list_options[$list['id']] = $list['title'];
        }

        $form['subscribers_list']['list_id'] = [
          '#type' => 'select',
          '#title' => $this->t('Subscriber List'),
          '#options' => $list_options,
          '#default_value' => $this->config('make.settings')->get('list_id'),
          '#required' => TRUE,
        ];
      } else {
        $this->messenger->addWarning($this->t('No subscriber lists found. Please check your User ID and API Key.'));
      }
    } catch (\Exception $e) {
      $this->messenger->addError($this->t('Authentication failed. Please check your User ID and API Key.'));
    }

    // Block Configuration Section
    $form['block_config'] = [
      '#type' => 'details',
      '#title' => $this->t('Signup Block Configuration'),
      '#group' => 'tabs',
      '#description' => $this->t('Use the checkboxes below to enable or disable specific fields that will appear in the subscription block. The <strong>Email</strong> field is always enabled & required.'),
      '#open' => TRUE,
    ];

    $enabled_fields = $this->config('make.settings')->get('enabled_fields') ?? [];

    $form['block_config']['enabled_fields'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Enable Fields'),
      '#options' => [
        'firstname' => $this->t('First Name'),
        'lastname' => $this->t('Last Name'),
        'address' => $this->t('Address'),
        'zip' => $this->t('ZIP'),
        'city' => $this->t('City'),
        'phone' => $this->t('Phone'),
        'company' => $this->t('Company'),
        'birthday' => $this->t('Birthday'),
        'tags' => $this->t('Tags'),
        'gender' => $this->t('Gender'),
      ],
      '#default_value' => $enabled_fields,
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Clear all existing messages
    $this->messenger->deleteAll();

    // Save the configuration settings
    $enabled_fields = array_filter($form_state->getValue('enabled_fields'));

    $this->config('make.settings')
      ->set('user_id', $form_state->getValue('user_id'))
      ->set('api_key', $form_state->getValue('api_key'))
      ->set('list_id', $form_state->getValue('list_id'))
      ->set('enabled_fields', $enabled_fields)
      ->save();

    // Re-validate the API key by trying to fetch the subscriber lists again, a lot of mess.
    try {
      $lists = $this->makeAsApiService->getSubscriberLists();

      if (!empty($lists)) {
        $this->messenger->addStatus($this->t('Authentication successful. Your subscriber lists have been retrieved.'));
      } else {
        $this->messenger->addWarning($this->t('No subscriber lists found. Please check your User ID and API Key.'));
      }
    } catch (\Exception $e) {
      $this->messenger->addError($this->t('Authentication failed. Please check your User ID and API Key.'));
    }

    parent::submitForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  protected function checkAccess() {
    return $this->currentUser()->hasPermission('administer make settings');
  }
}